/*
  Use this script to customize documentation lookups in CodeRunner.
  The script is loaded along with the web page when performing a lookup.
  The variable CodeRunnerDocsContext will contain information about the lookup.
  Most fields may also be used in the lookup URL, e.g. {query}, {lang}, etc
  Some important fields include:
  CodeRunnerDocsContext = {
    "query": the documentation search query,
    "lang": language name based on the current editor scope (if in doubt, use this),
    "languageName": active CodeRunner language name,
    "codeCompletionObject": if set, the code completion that invoked the lookup,
    "editorObject": if set, the editor context that invoked the lookup,
    "scope": the editor syntax scope for the current document
  }
  View the full contents of the variable using the code below:
    alert(JSON.stringify(CodeRunnerDocsContext, null, '\t'));

  You may also optionally define the function CodeRunnerDocsContextUpdate(),
  which will be called for consecutive lookups instead of reloading the entire
  page, given that the function returns true.
*/

document.addEventListener('DOMContentLoaded', function (e) {
	var firstQuestion = document.querySelector("[id^=question-summary]")
	if (firstQuestion) {
		window.location.hash = firstQuestion.getAttribute("id")
		var headers = document.getElementsByTagName("h1")
		if (headers.length > 0) {
			headers[0].dataset.coderunnerTocTitle = ""
		}
	} else {
		window.location.hash = "question"
		var answers = document.querySelectorAll("[data-answerid]")
		for (var i = 0; i < answers.length; i++) {
			answers[i].dataset.coderunnerTocTitle = "Answer #"+String(i+1)
			answers[i].dataset.coderunnerTocLevel = 2
		}
		if (answers.length > 0) {
			document.body.dataset.coderunnerTocTitle = String(answers.length) + " Answer" + (answers.length != 1 ? "s" : "")
		}
		var headers = document.querySelectorAll("h1, h2, h3, h4, h5, h6")
		for (var i = 0; i < headers.length; i++) {
			headers[i].dataset.coderunnerTocTitle = ""
		}
	}
})

